@@ -306,7 +306,8 @@ g_settings_schema_get_value (GSettingsSchema *schema,
   value = gvdb_table_get_raw_value (schema->priv->table, key);
 
   if G_UNLIKELY (value == NULL)
-    g_error ("schema does not contain a key named '%s'", key);
+    g_error ("Settings schema '%s' does not contain a key named '%s'",
+             schema->priv->name, key);
 
   iter = g_variant_iter_new (value);
   g_variant_unref (value);
